static char sccsid[160] = "@(#) samples/c/amqsvfcn.c, winnt, nt200, nt200-csd12          1.1.1.3  97/04/11 11:11:39";
 /********************************************************************/
 /*                                                                  */
 /* Program name: AMQSVFCN                                           */
 /*                                                                  */
 /* Description: Sample C skeleton of a Data Conversion exit         */
 /*              routine                                             */
 /*                                                                  */
 /* Licensed Materials - Property of IBM                             */
 /* 63H9336                                                          */
 /* (C) Copyright IBM Corp. 1994,1996.                               */
 /*                                                                  */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /* Function:                                                        */
 /*                                                                  */
 /*   AMQSVFCN is a sample C skeleton of a Data Conversion exit      */
 /*   routine.                                                       */
 /*   Each Data Conversion exit routine converts a single named      */
 /*   Format.  This skeleton is intended as a wrapper for code       */
 /*   fragments generated by the Data Conversion Exit-generation     */
 /*   utility program - crtmqcvx.                                    */
 /*                                                                  */
 /*   The utility produces one code fragment for each data           */
 /*   structure; a format may be defined in terms of several         */
 /*   such structures so that several code fragments will need       */
 /*   to be added to this skeleton to produce a routine to do        */
 /*   data conversion of the entire Format.                          */
 /*                                                                  */
 /*   Once complete the code should be compiled into a DLL with      */
 /*   the name of the DLL being the name of the format to be         */
 /*   converted. Note that this name should also be put into the     */
 /*   sample .DEF file AMQSVFCN.DEF - replace the library name       */
 /*   FORMAT1 with the name of the format being converted.           */
 /*   See the MQSeries Distributed Queue Management Guide for        */
 /*   further information.                                           */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /*  AMQSVFCN takes the parameters defined for a Data Conversion     */
 /*  exit routine in the CMQXC.H header file.                        */
 /*                                                                  */
 /********************************************************************/
 #include <cmqc.h>              /* For MQI datatypes                 */
 #include <cmqxc.h>             /* For MQI exit-related definitions  */
 #include <amqsvmha.h>          /* For sample macro definitions      */

 /********************************************************************/
 /* Insert the functions prototypes for the functions produced by    */
 /* the data conversion utility program.                             */
 /********************************************************************/

 MQDATACONVEXIT EntryPointFunctionName;

 /********************************************************************/
 /* The name of the function is not actually used to call the        */
 /* conversion exit BUT it is exported in the .DEF file              */
 /********************************************************************/
 void MQENTRY EntryPointFunctionName(
   PMQDXP   pDataConvExitParms, /* Data-conversion exit parameter    */
                                /* block                             */
   PMQMD    pMsgDesc,           /* Message descriptor                */
   MQLONG   InBufferLength,     /* Length in bytes of InBuffer       */
   PMQVOID  pInBuffer,          /* Buffer containing the unconverted */
                                /* message                           */
   MQLONG   OutBufferLength,    /* Length in bytes of OutBuffer      */
   PMQVOID  pOutBuffer)         /* Buffer containing the converted   */
                                /* message                           */
 {
   MQLONG   ReturnCode   = MQRC_NONE;
   MQHCONN  hConn        = pDataConvExitParms->Hconn;
   MQLONG   opts         = pDataConvExitParms->AppOptions;
   PMQBYTE  in_cursor    = pInBuffer;
   PMQBYTE  out_cursor   = pOutBuffer;
   PMQBYTE  in_lastbyte  = (PMQBYTE)pInBuffer + InBufferLength - 1;
   PMQBYTE  out_lastbyte = (PMQBYTE)pOutBuffer + OutBufferLength - 1;
   MQLONG   MsgEncoding  = pMsgDesc->Encoding;
   MQLONG   ReqEncoding  = pDataConvExitParms->Encoding;
   MQLONG   MsgCCSID     = pMsgDesc->CodedCharSetId;
   MQLONG   ReqCCSID     = pDataConvExitParms->CodedCharSetId;
   MQLONG   CompCode     = pDataConvExitParms->CompCode;
   MQLONG   Reason       = pDataConvExitParms->Reason;

   /******************************************************************/
   /* Insert calls to the code fragments to convert the format's     */
   /* structure(s) here.                                             */
   /* ReturnCode = ConverttagSTRUCT(                                 */
   /*                         &in_cursor,                            */
   /*                         &out_cursor,                           */
   /*                         in_lastbyte,                           */
   /*                         out_lastbyte,                          */
   /*                         hConn,                                 */
   /*                         opts,                                  */
   /*                         MsgEncoding,                           */
   /*                         ReqEncoding,                           */
   /*                         MsgCCSID,                              */
   /*                         ReqCCSID,                              */
   /*                         CompCode,                              */
   /*                         Reason);                               */
   /******************************************************************/


   /******************************************************************/
   /* Check whether the conversion succeeded or failed and return    */
   /* the values required by the caller.                             */
   /******************************************************************/
   if (ReturnCode == MQRC_NONE)
   {
     pDataConvExitParms->ExitResponse = MQXDR_OK;
     pDataConvExitParms->CompCode     = MQCC_OK;
     pDataConvExitParms->Reason       = ReturnCode;

     /****************************************************************/
     /* If the message had not been truncated then return its new    */
     /* length.                                                      */
     /* Warning - this assumes that out_cursor has been set up to    */
     /* point to the end of the message. Routines produced by the    */
     /* data conversion exit utility will do this BUT if you are     */
     /* writing your own routines ensure it is updated or the        */
     /* message could end up with a zero length and appear not to    */
     /* get converted!                                               */
     /****************************************************************/
     if (Reason != MQRC_TRUNCATED_MSG_ACCEPTED)
     {
       pDataConvExitParms->DataLength = out_cursor
                                         - (PMQBYTE)pOutBuffer;
     } /* end if */
   }
   /******************************************************************/
   /* If conversion failed for lack of data or lack of output buffer */
   /* but the message had been truncated then indicate success but   */
   /* do not adjust the values of Reason, CompCode or DataLength.    */
   /******************************************************************/
   else if ((Reason == MQRC_TRUNCATED_MSG_ACCEPTED) &&
            ((ReturnCode == MQRC_TRUNCATED_MSG_ACCEPTED) ||
             (ReturnCode == MQRC_CONVERTED_MSG_TOO_BIG)))
   {
     pDataConvExitParms->ExitResponse = MQXDR_OK;
   }
   /******************************************************************/
   /* Otherwise indicate that conversion of the message data failed. */
   /******************************************************************/
   else
   {
     pDataConvExitParms->ExitResponse = MQXDR_CONVERSION_FAILED;
     pDataConvExitParms->CompCode     = MQCC_WARNING;
     pDataConvExitParms->Reason       = ReturnCode;
   } /* end if */

   return;
 }

 /********************************************************************/
 /* Insert the functions produced by the data conversion exit        */
 /* utility program.                                                 */
 /********************************************************************/

 /********************************************************************/
 /*                                                                  */
 /* END OF AMQSVFCN                                                  */
 /*                                                                  */
 /********************************************************************/
